from typing import List, Dict
from copy import deepcopy
import numpy as np


def retrieve_nets_from_data_dict(data_dict: Dict,
                                 nets_by_sta: Dict[str, List[str]]) -> None:
    """
    Retrieve nets by sta_id from the given data_dict.

    :param data_dict: dict of data by station
    :param nets_by_sta: nets list by sta_id
    """
    for sta_id in data_dict.keys():
        if sta_id not in nets_by_sta:
            nets_by_sta[sta_id] = set()
        for c in data_dict[sta_id]:
            nets_by_sta[sta_id].update(
                data_dict[sta_id][c]['nets'])


def split_pegasus_vst_channel(selected_data_set_id: str, data_dict: Dict):
    """
    Each value consists of 4 bit.
    This function will split each bit into one array that will be the data
    of the channel VST<bit position> which has other keys' info from the
    original channel VST.
    The original VST channel is removed at the end of the function.

    :param selected_data_set_id: the id of the selected data set
    :param data_dict: data of the selected key
    """
    try:
        selected_data_dict = data_dict[selected_data_set_id]
    except KeyError:
        return
    vst_channels_data = [[], [], [], []]
    if 'VST' not in selected_data_dict:
        return
    data = selected_data_dict['VST']['tracesInfo'][0]['data']
    for d in data:
        b = '{0:04b}'.format(d)
        for i in range(4):
            vst_channels_data[i].append(int(b[i]))
    for i in range(4):
        name = f'VST{3 - i}'
        selected_data_dict[name] = deepcopy(
            selected_data_dict['VST'])
        selected_data_dict[name]['tracesInfo'][0]['data'] = np.array(
            vst_channels_data[i])
    del selected_data_dict['VST']


def split_q8_lft_channel(selected_data_set_id: str, data_dict: Dict):
    """
    Each data point in LFT encodes two values, the on-off state and the lock
    type of the GPS module.
    This function will split the LFT channel into two sub-channels, LFT0 and
    LFT1. LFT0 stores the GPS on-off state, while LFT1 stores the GPS lock
    type. The original data of the LFT channel is removed from the data.

    :param selected_data_set_id: the id of the selected data set
    :param data_dict: data of the selected key
    """
    try:
        selected_data_dict = data_dict[selected_data_set_id]
    except KeyError:
        return
    if 'LFT' not in selected_data_dict:
        return

    # We create a copy of the data first so that we can work directly on it
    # instead of having to create a (possibly very big) intermediate array.
    selected_data_dict['LFT0'] = deepcopy(selected_data_dict['LFT'])
    selected_data_dict['LFT1'] = deepcopy(selected_data_dict['LFT'])

    # The transformation below is based on page 218 of the Q8 Reference Guide,
    # Revision 5 (Q8RG‐R5‐20201207).
    # The on and off states are cleanly demarcated at the value 4.
    lft_0_data = selected_data_dict['LFT0']['tracesInfo'][0]['data']
    lft_0_data[np.where(lft_0_data <= 4)] = 0
    lft_0_data[np.where(lft_0_data > 4)] = 1

    # We have three fix types: no fix, unknown fix, and known fix (1D, 2D, 3D
    # fix).
    lft_1_data = selected_data_dict['LFT1']['tracesInfo'][0]['data']
    # We offset the final data by 20 to solve the problem where the final data
    # has overlap with the input data.
    lft_1_data[np.isin(lft_1_data, [1, 5])] = 20
    lft_1_data[np.isin(lft_1_data, [0, 6])] = 21
    lft_1_data[np.isin(lft_1_data, [2, 3, 4, 7, 8, 9])] = 22
    lft_1_data -= 20
    del selected_data_dict['LFT']
